@@ -117,6 +117,11 @@ ItemText::ItemText(const QString &text, const QString &richText, int maxLines, i
 {
     m_textDocument.setDefaultFont(font());
 
+    // Disable slow word wrapping initially.
+    QTextOption option = m_textDocument.defaultTextOption();
+    option.setWrapMode(QTextOption::NoWrap);
+    m_textDocument.setDefaultTextOption(option);
+
     setReadOnly(true);
     setUndoRedoEnabled(false);
     setTextInteractionFlags(
@@ -167,8 +172,6 @@ ItemText::ItemText(const QString &text, const QString &richText, int maxLines, i
     if (m_isRichText)
         sanitizeTextDocument(&m_textDocument);
 
-    setDocument(&m_textDocument);
-
     connect( this, &QTextEdit::selectionChanged,
              this, &ItemText::onSelectionChanged );
 }
@@ -224,6 +227,10 @@ void ItemText::updateSize(QSize maximumSize, int idealWidth)
         m_textDocument.setDefaultTextOption(option);
     }
 
+    // setDocument() is slow, so postpone this after resized properly
+    if (document() != &m_textDocument)
+        setDocument(&m_textDocument);
+
     const QRectF rect = m_textDocument.documentLayout()->frameBoundingRect(m_textDocument.rootFrame());
     setFixedWidth( static_cast<int>(rect.right()) );
 
