@@ -39,6 +39,7 @@
 
 #include <QApplication>
 #include <QDrag>
+#include <QElapsedTimer>
 #include <QKeyEvent>
 #include <QMimeData>
 #include <QProgressBar>
@@ -62,36 +63,6 @@ enum class MoveType {
     Relative
 };
 
-class UpdatesLocker final {
-public:
-    explicit UpdatesLocker(QWidget *widget)
-        : m_widget(widget)
-    {
-    }
-
-    UpdatesLocker(const UpdatesLocker &) = delete;
-    UpdatesLocker &operator =(const UpdatesLocker &) = delete;
-
-    ~UpdatesLocker()
-    {
-        if (m_updatesDisabled)
-            m_widget->setUpdatesEnabled(true);
-    }
-
-    void lock() {
-        if (m_updatesDisabled)
-            return;
-
-        m_updatesDisabled = m_widget->updatesEnabled();
-        if (m_updatesDisabled)
-            m_widget->setUpdatesEnabled(false);
-    }
-
-private:
-    QWidget *m_widget;
-    bool m_updatesDisabled = false;
-};
-
 /// Save drag'n'drop image data in temporary file (required by some applications).
 class TemporaryDragAndDropImage final : public QObject {
 public:
@@ -306,6 +277,7 @@ ClipboardBrowser::ClipboardBrowser(
     initSingleShotTimer( &m_timerEmitItemCount, 0, this, &ClipboardBrowser::emitItemCount );
     initSingleShotTimer( &m_timerUpdateSizes, 0, this, &ClipboardBrowser::updateSizes );
     initSingleShotTimer( &m_timerUpdateCurrent, 0, this, &ClipboardBrowser::updateCurrent );
+    initSingleShotTimer( &m_timerPreload, 0, this, &ClipboardBrowser::preloadCurrentPage );
 
     m_timerDragDropScroll.setInterval(20);
     connect( &m_timerDragDropScroll, &QTimer::timeout,
@@ -526,11 +498,15 @@ void ClipboardBrowser::updateEditorGeometry()
 void ClipboardBrowser::updateCurrentItem()
 {
     const QModelIndex current = currentIndex();
-    if ( current.isValid() && d.hasCache(current) ) {
-        ItemWidget *item = d.cache(current);
-        item->setCurrent(false);
-        item->setCurrent( hasFocus() );
-    }
+    if ( !current.isValid() )
+        return;
+
+    ItemWidget *item = d.cacheOrNull(current.row());
+    if (!item)
+        return;
+
+    item->setCurrent(false);
+    item->setCurrent( hasFocus() );
 }
 
 QModelIndex ClipboardBrowser::indexNear(int offset) const
@@ -566,8 +542,6 @@ void ClipboardBrowser::connectModelAndDelegate()
     connect( &m, &QAbstractItemModel::dataChanged,
              &d, &ItemDelegate::dataChanged );
 
-    connect( &m, &QAbstractItemModel::dataChanged,
-             this, &ClipboardBrowser::onDataChanged );
     connect( &m, &QAbstractItemModel::rowsInserted,
              this, &ClipboardBrowser::onRowsInserted);
 
@@ -634,20 +608,55 @@ int ClipboardBrowser::findPreviousVisibleRow(int row)
     return row >= 0 ? row : -1;
 }
 
+void ClipboardBrowser::preloadCurrentPage()
+{
+    const int h = viewport()->contentsRect().height();
+    const QModelIndex start = indexNear(0);
+    preload(h, false, start);
+}
+
+void ClipboardBrowser::preloadCurrentPageLater()
+{
+    if ( !m_timerPreload.isActive() )
+        m_timerPreload.start();
+}
+
 void ClipboardBrowser::preload(int pixels, bool above, const QModelIndex &start)
 {
-    const int s = 2 * spacing();
+    QElapsedTimer t;
+    t.start();
+
+    const int s = spacing();
     const int direction = above ? -1 : 1;
     int row = start.row();
-    QModelIndex ind = index(row);
+    QModelIndex ind = start;
     int y = 0;
 
-    for ( ; ind.isValid() && y < pixels; ind = index(row) ) {
-        if ( !isRowHidden(row) ) {
-            d.cache(ind);
-            y += s + d.sizeHint(ind).height();
+    const auto margins = m_sharedData->theme.margins();
+    const auto rowNumberSize = m_sharedData->theme.rowNumberSize();
+    const auto padding = QPoint(rowNumberSize.width() + margins.width() - s, margins.height() - s);
+
+    int items = 0;
+    bool anyShown = false;
+    for ( ; ind.isValid() && y < pixels; row += direction, ind = index(row) ) {
+        if ( isRowHidden(row) )
+            continue;
+
+        const auto rect = visualRect(ind);
+        const auto pos = rect.topLeft() + padding;
+
+        if ( d.showAt(ind, pos) )
+            anyShown = true;
+
+        if (items > 0)
+            y += rect.height();
+
+        ++items;
+        if (anyShown && items > 1 && t.elapsed() > 20) {
+            // Preloading takes too long, preload rest of the items later.
+            preloadCurrentPageLater();
+            return;
         }
-        row += direction;
     }
 }
 
@@ -883,11 +892,6 @@ QPixmap ClipboardBrowser::renderItemPreview(const QModelIndexList &indexes, int
     return pix;
 }
 
-void ClipboardBrowser::onDataChanged(const QModelIndex &, const QModelIndex &)
-{
-    emit itemsChanged(this);
-}
-
 void ClipboardBrowser::onRowsInserted(const QModelIndex &, int first, int last)
 {
     QModelIndex current;
@@ -979,7 +983,7 @@ void ClipboardBrowser::resizeEvent(QResizeEvent *event)
 
 void ClipboardBrowser::showEvent(QShowEvent *event)
 {
-    if ( m.rowCount() > 0 && !d.hasCache(index(0)) )
+    if ( m.rowCount() > 0 && d.cacheOrNull(0) == nullptr )
         scrollToTop();
 
     QListView::showEvent(event);
@@ -1022,8 +1026,10 @@ void ClipboardBrowser::focusInEvent(QFocusEvent *event)
         m_itemSaver->setFocus(true);
 }
 
-void ClipboardBrowser::focusOutEvent(QFocusEvent *)
+void ClipboardBrowser::focusOutEvent(QFocusEvent *event)
 {
+    QListView::focusOutEvent(event);
+
     if (m_itemSaver)
         m_itemSaver->setFocus(false);
 }
@@ -1087,9 +1093,9 @@ void ClipboardBrowser::paintEvent(QPaintEvent *e)
         }
     }
 
-    const bool canUpdate = updatesEnabled();
+    preloadCurrentPage();
+
     QListView::paintEvent(e);
-    setUpdatesEnabled(canUpdate);
 
     // If dragging an item into list, draw indicator for dropping items.
     if (m_dragTargetRow != -1) {
@@ -1196,7 +1202,7 @@ void ClipboardBrowser::mouseMoveEvent(QMouseEvent *event)
         if (target == this || target == viewport()) {
             moveIndexes(indexesToRemove, m_dragTargetRow, &m, MoveType::Absolute);
         } else if ( target && target->window() == window()
-                    && m_itemSaver->canMoveItems(selected) )
+                    && m_itemSaver->canRemoveItems(selected, nullptr) )
         {
             removeIndexes(selected);
         }
@@ -1448,10 +1454,7 @@ void ClipboardBrowser::keyPressEvent(QKeyEvent *event)
         const int h = viewport()->contentsRect().height();
 
         // Preload next and previous pages so that up/down and page up/down keys scroll correctly.
-        {
-            UpdatesLocker locker(this);
-            locker.lock();
-
+        if ( !m_timerPreload.isActive() ) {
             if (key == Qt::Key_PageDown || key == Qt::Key_PageUp)
                 preload(h, (key == Qt::Key_PageUp), current);
             else if (key == Qt::Key_Down || key == Qt::Key_Up)
@@ -1516,6 +1519,11 @@ void ClipboardBrowser::keyPressEvent(QKeyEvent *event)
         const QItemSelectionModel::SelectionFlags flags = selectionCommand(index(row), event);
         const bool setCurrentOnly = flags.testFlag(QItemSelectionModel::NoUpdate);
         const bool keepSelection = setCurrentOnly || flags.testFlag(QItemSelectionModel::SelectCurrent);
+
+        // Use more stable page scrolling by keeping current item at top.
+        if (key == Qt::Key_PageDown || key == Qt::Key_PageUp)
+            scrollTo( index(row), QAbstractItemView::PositionAtTop );
+
         setCurrent(row, keepSelection, setCurrentOnly);
         break;
     }
@@ -1655,8 +1663,6 @@ bool ClipboardBrowser::add(const QVariantMap &data, int row)
     const int newRow = row < 0 ? m.rowCount() : qMin(row, m.rowCount());
     m.insertItem(data, newRow);
 
-    delayedSaveItems();
-
     return true;
 }
 
@@ -1842,21 +1848,14 @@ void ClipboardBrowser::move(int key)
     scrollTo( currentIndex() );
 }
 
-QWidget *ClipboardBrowser::currentItemPreview()
+QWidget *ClipboardBrowser::currentItemPreview(QWidget *parent)
 {
     if (!isLoaded())
         return nullptr;
 
     const QModelIndex index = currentIndex();
-    const bool antialiasing = m_sharedData->theme.isAntialiasingEnabled();
     const auto data = itemData(index);
-    ItemWidget *itemWidget =
-            m_sharedData->itemFactory->createItem(data, this, antialiasing, false, true);
-    QWidget *w = itemWidget->widget();
-
-    d.highlightMatches(itemWidget);
-
-    return w;
+    return d.createPreview(data, parent);
 }
 
 void ClipboardBrowser::findNext()
@@ -1871,12 +1870,6 @@ void ClipboardBrowser::findPrevious()
         m_editor->findPrevious(d.searchExpression());
 }
 
-void ClipboardBrowser::updateItemWidget(const QModelIndex &index)
-{
-    setUpdatesEnabled(false);
-    d.cache(index);
-}
-
 bool ClipboardBrowser::isInternalEditorOpen() const
 {
     return !m_editor.isNull();
