@@ -152,6 +152,11 @@ public:
      */
     ClipboardBrowser *browser();
 
+    /**
+     * Return browser widget in current tab or nullptr if not loaded.
+     */
+    ClipboardBrowser *browserOrNull();
+
     /** Return browser containing item or nullptr. */
     ClipboardBrowser *browserForItem(const QModelIndex &index);
 
@@ -269,8 +274,7 @@ public:
 
     void setTrayTooltip(const QString &tooltip);
 
-    QStringList menuItemMatchCommands(int actionId);
-    bool setMenuItemEnabled(int actionId, int menuItemMatchCommandIndex, bool enabled);
+    bool setMenuItemEnabled(int actionId, int currentRun, int menuItemMatchCommandIndex, bool enabled);
 
     QVariantMap setDisplayData(int actionId, const QVariantMap &data);
 
@@ -286,6 +290,8 @@ public:
     /** Load settings. */
     void loadSettings();
 
+    void loadTheme(const QSettings &themeSettings);
+
     /** Open help. */
     void openHelp();
 
@@ -418,6 +424,8 @@ signals:
 
     void disableClipboardStoringRequest(bool disable);
 
+    void sendActionData(int actionId, const QByteArray &bytes);
+
 protected:
     void keyPressEvent(QKeyEvent *event) override;
     void keyReleaseEvent(QKeyEvent *event) override;
@@ -459,9 +467,12 @@ private:
 
     void updateContextMenuTimeout();
 
-    void updateTrayMenuTimeout();
+    void updateTrayMenuItemsTimeout();
+
+    void updateItemPreviewAfterMs(int ms);
+
+    void updateItemPreviewTimeout();
 
-    void updateItemPreview();
     void setItemPreviewVisible(bool visible);
 
     /** Update icon snip animation. */
@@ -496,13 +507,12 @@ private:
 
     void onItemWidgetCreated(const PersistentDisplayItem &item);
 
-    void onDisplayActionFinished();
-
     void onActionDialogAccepted(const Command &command, const QStringList &arguments, const QVariantMap &data);
 
     void updateEnabledCommands();
 
     void updateCommands(QVector<Command> allCommands, bool forceSave);
+    bool addPluginCommands(QVector<Command> *allCommands);
 
     void disableHideWindowOnUnfocus();
     void enableHideWindowOnUnfocus();
@@ -517,6 +527,7 @@ private:
     };
 
     struct MenuMatchCommands {
+        int currentRun = 0;
         int actionId = -1;
         QStringList matchCommands;
         QVector< QPointer<QAction> > actions;
@@ -541,9 +552,8 @@ private:
 
     void updateContextMenu(int intervalMsec);
 
-    void updateTrayMenu();
-
-    void updateTrayMenuClipboard();
+    void updateTrayMenuItems();
+    void updateTrayMenuCommands();
 
     void updateNotifications();
 
@@ -589,6 +599,10 @@ private:
     void addCommandsToTrayMenu(const QVariantMap &clipboardData);
     void addMenuMatchCommand(MenuMatchCommands *menuMatchCommands, const QString &matchCommand, QAction *act);
     void runMenuCommandFilters(MenuMatchCommands *menuMatchCommands, const QVariantMap &data);
+    void interruptMenuCommandFilters(MenuMatchCommands *menuMatchCommands);
+    void stopMenuCommandFilters(MenuMatchCommands *menuMatchCommands);
+
+    void terminateAction(int *actionId);
 
     bool isItemMenuDefaultActionValid() const;
 
@@ -626,7 +640,6 @@ private:
 
     QMenu *m_menuItem;
     TrayMenu *m_trayMenu;
-    QPointer<QAction> m_trayMenuClipboardAction;
 
     QSystemTrayIcon *m_tray;
 
@@ -647,7 +660,8 @@ private:
 
     QTimer m_timerUpdateFocusWindows;
     QTimer m_timerUpdateContextMenu;
-    QTimer m_timerUpdateTrayMenu;
+    QTimer m_timerUpdateTrayMenuItems;
+    QTimer m_timerUpdatePreview;
     QTimer m_timerTrayAvailable;
     QTimer m_timerTrayIconSnip;
     QTimer m_timerSaveTabPositions;
@@ -676,11 +690,12 @@ private:
     bool m_activatingItem = false;
 
     QVector< QPointer<QAction> > m_actions;
+    QPointer<QAction> m_showHideAction;
     MenuItems m_menuItems;
 
     QList<PersistentDisplayItem> m_displayItemList;
     PersistentDisplayItem m_currentDisplayItem;
-    QPointer<Action> m_currentDisplayAction;
+    int m_displayActionId = -1;
 
     MenuMatchCommands m_trayMenuMatchCommands;
     MenuMatchCommands m_itemMenuMatchCommands;
