@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2019, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -44,7 +44,7 @@
 
 namespace {
 
-class NotificationButtonWidget : public QPushButton
+class NotificationButtonWidget final : public QPushButton
 {
     Q_OBJECT
 
