@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2018, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2019, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -151,7 +151,7 @@ QLabel *createLabel(const QString &objectName, QWidget *parent)
     return label;
 }
 
-class ItemLabel : public QWidget {
+class ItemLabel final : public QWidget {
 public:
     explicit ItemLabel(QTreeWidgetItem *item)
         : QWidget(item->treeWidget())
